@@ -10,6 +10,8 @@ import platform
 OS = platform.system() #'win32' , linux, #Darwin
 
 
+
+
 if OS == 'Darwin':
     import matplotlib
     matplotlib.use("TkAgg")
@@ -23,13 +25,16 @@ else:
     from .settings.personal import CRUNCHER, PATH_TO_DATASHELF, DB_READ_ONLY, DEBUG
   
 
+
+ 
+
 #%% general setup
 
 
 REQUIRED_META_FIELDS = {'entity',
                         'scenario',
-                        'institution',
-                        'year',
+                        'source_name',
+                        'source_year',
                         'unit'}
 
 OPTIONAL_META_FIELDS = ['category',
