@@ -56,6 +56,32 @@ except:
         print('use: "pip install hdx-python-country" to install')
         return None
 
+def colorGenerator(reset=False):
+    
+    
+    _colors = ['#1f77b4',
+            '#aec7e8', #aec7e8
+            '#ff7f0e', #ff7f0e
+            '#ffbb78', #ffbb78
+            '#2ca02c', #2ca02c
+            '#98df8a', #98df8a
+            '#d62728', #d62728
+            '#ff9896', #ff9896
+            '#9467bd', #9467bd
+            '#c5b0d5', #c5b0d5
+            '#8c564b', #8c564b
+            '#c49c94', #c49c94
+            '#e377c2', #e377c2
+            '#f7b6d2', #f7b6d2
+            '#7f7f7f', #7f7f7f
+            '#c7c7c7', #c7c7c7
+            '#bcbd22', #bcbd22
+            '#dbdb8d', #dbdb8d
+            '#17becf', #17becf
+            '#9edae5', #9edae5
+            ]
+    for color in _colors:
+        yield(color)
 
  
 #%%
@@ -186,6 +212,14 @@ def convertToCO2eq(dataTable, context = 'GWP100_AR5'):
     
     return outTable
 
+def isUnit(unit):
+    try:
+        core.getUnit(unit)
+    except:
+        return False
+    else:
+        return True
+
 def scatterIndicatorComparison(tableX, tableY):
     timeCol = list(set(tableY.columns).intersection(set(tableX.columns)))
     for ISOcode in tableX.index:
@@ -340,6 +374,16 @@ def convertIndexToISO(table):
   
     return table
 
+def addCountryNames(table):
+    names = list()
+    for idx in table.index:
+        if idx in dt.mapp.countries.codes.index:
+            names.append(dt.mapp.countries.codes.loc[idx,'name'])
+        else:
+            names.append('')
+    table.loc[:,'country_name'] = names
+    return table
+
 def getCountryExtract(countryList, sourceList='all'):
     
     if sourceList == 'all':
@@ -397,7 +441,7 @@ def _res2Excel(resFull, countryList):
                 i = i+1
         
         outDf = outDf.loc[:,~outDf.isnull().all(axis=0)]
-        outDf.to_excel(config.MODULE_PATH + 'extracts/' + coISO + '.xlsx')
+        outDf.to_excel('extract_'+  coISO + '.xlsx')
         
         
     #%%
@@ -503,6 +547,9 @@ def update_DB_from_folderV3(folderToRead, message=None, cleanTables=True):
 
             table = dt.read_csv(os.path.join(folderToRead, file))
             source = table.meta['source']
+            
+#            if not 'Emissions|CO2|Industrial' in table.ID:
+#                continue
             if source in tablesToUpdate.keys():
     
     
@@ -517,7 +564,7 @@ def update_DB_from_folderV3(folderToRead, message=None, cleanTables=True):
         for source in tablesToUpdate.keys():
     
             tablesToUpdate[source] = metaV2_to_meta_V3(tablesToUpdate[source])
-    #    return tablesToUpdate
+#        return tablesToUpdate
     
         for source in tablesToUpdate.keys():
             sourceMetaDict = dict()
@@ -525,7 +572,50 @@ def update_DB_from_folderV3(folderToRead, message=None, cleanTables=True):
             core.DB.commitTables(tablesToUpdate[source], 
                             message = message,
                             sourceMetaDict = sourceMetaDict, 
-                            cleanTables=cleanTables)
+                            cleanTables=cleanTables,
+                            update=True)
+
+#def update_DB_from_folderV3(folderToRead, message=None, cleanTables=True):
+#    import math
+#    fileList = os.listdir(folderToRead)
+#    fileList = [file for file in fileList if '.csv' in file[-5:].lower()]
+#
+#    
+#
+##    filesPerCommit = 5000
+##    nCommits = math.ceil((len(fileList))/filesPerCommit)
+##    for nCommit in range(nCommits):
+#    tablesToUpdate = dict()
+#    for file in fileList:
+#
+#        table = dt.read_csv(os.path.join(folderToRead, file))
+#        source = table.meta['source']
+#        
+#        if not 'Emissions|CO2|Industrial' in table.ID:
+#                continue
+#        if source in tablesToUpdate.keys():
+#
+#
+#            tablesToUpdate[source].append(table)
+#        else:
+#            tablesToUpdate[source] = [table]
+#    
+##    if message is None:
+##
+##        message = 'External data added from external source by ' + config.CRUNCHER + '{}/{}'.format(nCommit,nCommits)
+#
+#    for source in tablesToUpdate.keys():
+#
+#        tablesToUpdate[source] = metaV2_to_meta_V3(tablesToUpdate[source])
+#    return tablesToUpdate
+    
+#        for source in tablesToUpdate.keys():
+#            sourceMetaDict = dict()
+#            sourceMetaDict['SOURCE_ID']= source
+#            core.DB.commitTables(tablesToUpdate[source], 
+#                            message = message,
+#                            sourceMetaDict = sourceMetaDict, 
+#                            cleanTables=cleanTables)
 
 def metaV2_to_meta_V3(tableSet):
     replacementDict = {#'Capacity': 'Electricity|capacity',
@@ -563,14 +653,14 @@ def metaV2_to_meta_V3(tableSet):
                    'Emissions|PFCS' : 'Emissions|PFCs',
                    'Electricity_output'  : 'Electricity_generation',
                    'Electricity_output ' : 'Electricity_generation',
-                   'Elect_Capacity'     : 'Electricity|capacity',  
-                   'Electrical_capacity' : 'Electricity|capacity',
-                   'Electrical|capacity' : 'Electricity|capacity',
-                   'Electricity|capacity ': 'Electricity|capacity',
-                   'Electricity_generation' : 'Electricity|generation',
-                   'Electricity_genertation ' : 'Electricity|generation', 
-                   'Elect_Generation' : 'Electricity|generation',
-                   'Electricity_and_heat_generation' : 'Electricity&Heat|generation',
+                   'Elect_Capacity'     : 'Electricity_capacity',  
+                   'Electrical_capacity' : 'Electricity_capacity',
+                   'Electrical|capacity' : 'Electricity_capacity',
+                   'Electricity|capacity ': 'Electricity_capacity',
+                   'Electricity»generation' : 'Electricity_generation',
+                   'Electricity»genertation ' : 'Electricity_generation', 
+                   'Elect_Generation' : 'Electricity_generation',
+                   'Electricity_and_heat_generation' : 'Electricity&Heat_generation',
                    'Price_' : 'Price|',
                    'Primary_Energy_': 'Primary_Energy|',
                    'Primary_energy' : 'Primary_Energy',
@@ -579,7 +669,7 @@ def metaV2_to_meta_V3(tableSet):
                    'Transfers_' : 'Transfers|',
                    'Total_PE_supply_' : 'Total_PE_supply|',
                    'Total_consumption_' : 'Total_consumption|',
-                   'Emissions_er_capita' : 'Emissions_per_capita'}
+                   'Emissions_per_capita' : 'Emissions_per_capita'}
 
     entityList = ['Electricity|generation|',
                   'Electricity|capacity|',
@@ -779,6 +869,13 @@ def yearsColumnsOnly(dataframe):
     for col in dataframe.columns:
         if REG_YEAR.search(str(col)) is not None:
             newColumns.append(col)
+        else:
+            try: 
+                if ~np.isnan(col) and REG_YEAR.search(str(int(col))) is not None:
+                #   test float string
+                    newColumns.append(col)
+            except:
+                pass
     return newColumns
 
 import csv
@@ -800,6 +897,63 @@ def csv_to_dict(filePath):
 #            v = rows[1]
             mydict[row[0]] =  row[1]
     return mydict
+
+def aggregate_table_to_region(table, mapping):
+    #TODO remove soon
+    raise(Warning('Depricated soon. Please use new implementaion in datatoolbox.tools.for_datatables'))
+    
+    missingCountryDict = dict()
+    
+    for region in mapping.listAll():
+
+        
+        missingCountries = set(mapping.membersOf(region)) - set(table.index)
+#                print('missing countries: {}'.format(missingCountries))
+        missingCountryDict[region] = list(missingCountries)
+        availableCountries = set(mapping.membersOf(region)).intersection(table.index)
+        if len(availableCountries) >0:
+            table.loc[region,:] = table.loc[availableCountries,:].sum(axis=0, skipna=True)
+
+    return table, missingCountryDict
+
+def aggregate_tableset_to_region(tableSet, mapping):
+    missingCountryDf = pd.DataFrame(columns=mapping.listAll())
+
+    for tableKey in tableSet.keys():
+
+        for region in mapping.listAll():
+#                print(region)
+            
+            missingCountries = set(mapping.membersOf(region)) - set(tableSet[tableKey].index)
+#                print('missing countries: {}'.format(missingCountries))
+            missingCountryDf.loc[tableSet[tableKey].ID, region] = list(missingCountries)
+            availableCountries = set(mapping.membersOf(region)).intersection(tableSet[tableKey].index)
+            if len(availableCountries) >0:
+                tableSet[tableKey].loc[region,:] = tableSet[tableKey].loc[availableCountries,:].sum(axis=0, skipna=True)
+
+    return tableSet, missingCountryDf
+
+
+import matplotlib.pylab as plt
+
+def plotTables(tableList, countryList):
+    fig = plt.figure(1)
+    plt.clf()
+    NUM_COLORS = len(countryList)
+    
+    cm = plt.get_cmap('gist_rainbow')
+    coList = [cm(1.*i/NUM_COLORS) for i in range(NUM_COLORS)]
+#    fig = plt.figure()
+#    ax = fig.add_subplot(111)
+#    ax.set_color_cycle([cm(1.*i/NUM_COLORS) for i in range(NUM_COLORS)])
+
+#    ax.set_color_cycle([cm(1.*i/NUM_COLORS) for i in range(NUM_COLORS)])
+    for table in tableList:
+        for i, coISO in enumerate(countryList):
+            plt.plot(table.columns, table.loc[coISO,:].T, color =coList[i] )
+    plt.legend(countryList)
+    plt.title(table.ID)
+    
 #%%    
 if __name__ == '__main__':
     #%%
@@ -832,4 +986,6 @@ if __name__ == '__main__':
                            "author" : 'AG'})
         return table
 
-    outputTables, success = forAll(calculateTotalBiomass, "scenario")
\ No newline at end of file
+    outputTables, success = forAll(calculateTotalBiomass, "scenario")
+    
+    
\ No newline at end of file
